(function(window, $) {

    var SkeletonCart = {
        api_base: '/api',
        app_base: '',
        cart_path: '/cart',
        credentials: true,
        make_path: function(paths) {
            var url = paths.join('/');
            var m = url.match(/(https?:\/\/[^/]+)?(.*)/);
            if (m) {
              var host = m[1] || '';
              var path = m[2].replace(/\/{2,}/g, '/');
              url = host + path;
            }
            return url;
        },
        add: function(params, options) {
            $.ajax({
                type: 'POST',
                url: this.make_path([this.app_base, this.api_base, this.cart_path, 'add']),
                data: params,
                success: null,
                dataType: 'json',
                xhrFields: {
                    withCredentials: this.credentials,
                }
            }).done(function(data, textStatus, jqXHR) {
                if(options != undefined) {
                    if(options.onAdd != undefined) {
                        options.onAdd(data);
                    }
                    else if(options.onSuccess != undefined) {
                        options.onSuccess(data);
                    }
                }
            }).fail(function(jqXHR, textStatus, errorThrown) {
                if(options != undefined && options.onError != undefined) {
                    var data = jqXHR.responseJSON;
                    if(!data) {
                        data = {
                            success: false,
                            error: errorThrown,
                            result: null
                        };
                    }
                    options.onError(data, 'add');
                }
            }).always(function() {
                if(options != undefined && options.onFinish != undefined) {
                    options.onFinish();
                }
            });
        },

        count: function(options) {
            $.ajax({
                type: 'GET',
                url: this.make_path([this.app_base, this.api_base, this.cart_path, 'count']),
                data: '',
                success: null,
                dataType: 'json',
                xhrFields: {
                    withCredentials: this.credentials,
                },
            }).done(function(data, textStatus, jqXHR) {
                if(options != undefined && options.onSuccess != undefined) {
                    options.onSuccess(data);
                }
            }).fail(function(jqXHR, textStatus, errorThrown) {
                if(options != undefined && options.onError != undefined) {
                    options.onError(jqXHR.responseJSON, 'count');
                }
            }).always(function() {
                if(options != undefined && options.onFinish != undefined) {
                    options.onFinish();
                }
            });
        },

        loadItem: function(id, options) {
            $.ajax({
                type: 'GET',
                url: this.make_path([this.app_base, this.api_base, 'item/get', id]),
                data: '',
                success: null,
                dataType: 'json',
                xhrFields: {
                    withCredentials: this.credentials,
                },
            }).done(function(data, textStatus, jqXHR) {
                if(options != undefined) {
                    if(options.onLoadItem != undefined) {
                        options.onLoadItem(data);
                    }
                    else if(options.onSuccess != undefined) {
                        options.onSuccess(data);
                    }
                }
            }).fail(function(jqXHR, textStatus, errorThrown) {
                if(options != undefined && options.onError != undefined) {
                    options.onError(jqXHR.responseJSON, 'loadItem');
                }
            }).always(function() {
                if(options != undefined && options.onFinish != undefined) {
                    options.onFinish();
                }
            });
        },

        tpc_supported: null,

        checkTPC: function(options) {
            var self = this;

            var frame = $('<iframe>', {
                id: 'tpc',
                src: self.make_path([self.app_base, '/tpc/start']),
                style: 'display: none; position: fixed'
            });
            $(window).on('message', function(ev) {
                var event = ev.originalEvent;
                if(event.data === 'tpc.supported' || event.data === 'tpc.unsupported') {
                    self.tpc_supported = event.data === 'tpc.supported';
                    frame.remove();
                    $(window).off('message');
                    if(options != undefined && options.onCheckTPC != undefined) {
                        options.onCheckTPC(self.tpc_supported, false);
                    }
                }
            });

            setTimeout(function() {
                if(self.tpc_supported === null) {
                    self.tpc_supported = false;
                    frame.remove();
                    if(options != undefined && options.onCheckTPC != undefined) {
                        options.onCheckTPC(self.tpc_supported, true);
                    }
                }
            }, 3000);

            frame.appendTo($('body'));
        }
    };

    if(window.SkeletonCartSettings != undefined) {
        if(window.SkeletonCartSettings.app_base !== undefined) {
            SkeletonCart.app_base = window.SkeletonCartSettings.app_base;
        }
        if(window.SkeletonCartSettings.cart_path !== undefined) {
            SkeletonCart.cart_path = window.SkeletonCartSettings.cart_path;
        }
        if(window.SkeletonCartSettings.onCheckTPC) {
            SkeletonCart.checkTPC({ onCheckTPC: window.SkeletonCartSettings.onCheckTPC });
        }
    }

    var methods = {
        init: function(options) {

            return this.each(function() {
                var $this = $(this);
                var settings = $.extend({
                    onInit: function() {
                        methods.loadItem.apply($this);
                    },
                    onBeforeSubmit: function() {},
                    onAfterSubmit: function() {},
                    onAdd: function(data) {},
                    onSuccess: function(data) {},
                    onError: function(data, method) {},
                    onFinish: function() {},
                    onLoadItem: function(data) {},
                    onGetItemId: function() {
                        var id;
                        var $id = $('[name="id"]', this);
                        var type = $id.attr('type');
                        if(type === 'radio' || type === 'checkbox') {
                            id = $id.filter(':checked').val();
                        }
                        else {
                            id = $id.val();
                        }
                        return id;
                    },
                    loadItemData: false,
                }, options);
                $.each(settings, function(key, value) {
                    if(typeof value === 'function') {
                        settings[key] = $.proxy(value, $this);
                    }
                });
                var data = $this.data('scItem');
                if(!data) {
                    $this.data('scItem', settings);
                }
                $this.on('submit.scItem', $.proxy(methods.submit, $this));
                if(settings.onInit != undefined) {
                    settings.onInit.apply($this);
                }
            });
        },
        submit: function() {
            var settings = this.data('scItem');
            if(settings.onBeforeSubmit != undefined) {
                settings.onBeforeSubmit.apply(this);
            }

            var data = this.serialize();
            SkeletonCart.add(data, settings);

            if(settings.onAfterSubmit != undefined) {
                settings.onAfterSubmit.apply(this);
            }
            return false;
        },
        loadItem: function() {
            var settings = this.data('scItem');
            var id = settings.onGetItemId.apply(this);
            if(!id) {
                return;
            }
            if(settings.loadItemData) {
                SkeletonCart.loadItem(id, settings);
            }
        }
    };

    $.fn.scItem = function(method) {
        if(methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        }
        else if(typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        }
        else {
            $.error('Method ' + method + ' does not exist on jQuery.scItem');
        }
    };

    var itemCountMethods = {
        init: function(options) {
            var settings = $.extend({
                onUpdate: itemCountMethods.onUpdate
            }, options);

            return this.each(function () {
                var $this = $(this),
                    data = $this.data('scItemCount');
                if(!data) {
                    $this.data('scItemCount', settings);
                }
                $this.scItemCount('update');
            })
        },
        update: function() {
            var self = this;
            var $this = $(this);
            var settings = $this.data('scItemCount');
            SkeletonCart.count({
                onSuccess: function(data) {
                    settings.onUpdate.apply(self, [ data.result.count ]);
                }
            });
        },
        onUpdate: function(count) {
            var $this = $(this);
            if(count != undefined && count > 0) {
                $this.text(count);
            }
        }
    };

    $.fn.scItemCount = function(method) {
        if(itemCountMethods[method]) {
            return itemCountMethods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        }
        else if(typeof method === 'object' || !method) {
            return itemCountMethods.init.apply(this, arguments);
        }
        else {
            $.error('Method ' + method + ' does not exist on jQuery.scItemCount');
        }
    };

    window.SkeletonCart = SkeletonCart;

})(window, jQuery);

/* vim: set et ts=4 sts=4 sw=4: */
