<?php

namespace App\API;

class CartController extends \App\CartController
{
    public function process()
    {
        try {
            switch ($this->mode) {
                case 'show':
                    $this->show();
                    break;
                case 'add':
                    $this->addItem();
                    break;
                case 'remove':
                    $this->removeItem();
                    break;
                case 'update':
                    $this->updateItem();
                    break;
                case 'count':
                    $this->totalAmount();
                    break;
                default:
                    $this->json_error('不正なパスです。', null, 404);
                    break;
            }
        } catch (\App\CartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $this->json_error($e->getMessage());
        } catch (\Exception $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $this->json_error($e->getMessage(), null, 500);
        }
    }

    public function json_result($result)
    {
        $this->json($result);
    }

    public function json_success($result = null)
    {
        $this->json(array(
            'success' => true,
            'error' => null,
            'result' => $result,
        ));
    }

    public function json_error($error, $result = null, $code = 400)
    {
        $this->json(array(
            'success' => false,
            'error' => $error,
            'result' => $result,
        ), $code);
    }

    public function redirect_to_cart()
    {
        $this->json(array(
            'result' => true
        ));
    }

    public function render($template, $data, $code = 200)
    {
        if($code == 200) {
            $this->json_success($data);
        }
        else {
            $error = 'エラーが発生しました。';
            if($template == '_maintenance.html') {
                $error = 'メンテナンス中です。';
            }
            $this->json_error($error, $data, $code);
        }
    }

    public function getTemplateParams()
    {
        $cart = \App::cart();
        $config = \App::config();
        $data = array(
            'count' => $cart->totalAmount(),
            'items' => $cart->getItemsArray(),
            'subtotal' => $cart->subTotalPrice(),
            'tax' => $cart->tax(),
            'total_weight' => $cart->totalWeight(),
            'weight' => $cart->totalWeight(),
            'free_delivery_price' => $config->getFreePostagePrice(),
            'payment' => $cart->payment(),
            'tax_rate' => $config->getTaxRate(),
        );
        $labels = $config->cartItemOptionLabels();
        $data = array_merge($data, $labels);

        if ($cart->hasError()) {
            $errors = array();
            if ($cart->hasError()) {
                $errors[] = $cart->error();
                foreach ($cart as $cartItem) {
                    if ($cartItem->hasError()) {
                        $errors[] = $cartItem->error();
                    }
                }
            }
            if(!empty($errors)) {
                $data['errors'] = $errors;
            }
        }

        return $data;
    }

    public function show()
    {
        $service = \App::cartService();
        $service->checkCart();
        $data = $this->getTemplateParams();
        $this->json_success($data);
    }

    public function addItem()
    {
        if (\App::request()->method !== 'POST') {
            throw new \RuntimeException('Invalid Method');
        }
        $params = $this->getAddParams();
        $service = \App::cartService();
        $service->addItem($params, true);
        $this->json_success();
    }

    public function removeItem()
    {
        if (\App::request()->method !== 'POST') {
            throw new \RuntimeException('Invalid Method');
        }
        $params = $this->getRemoveParams();
        $service = \App::cartService();
        $service->removeItem($params);
        $this->json_success();
    }

    public function updateItem()
    {
        if (\App::request()->method !== 'POST') {
            throw new \RuntimeException('Invalid Method');
        }
        $params = $this->getUpdateParams();
        $service = \App::cartService();
        try {
            $service->updateItem($params, true);
        } catch (ValidationException $e) {
            $errors = $e->getErrors();
        }
        if (isset($errors)) {
            $this->json_error($e->getMessage(), $errors);
        } else {
            $this->json_success();
        }
    }

    public function totalAmount()
    {
        $service = \App::cartService();
        $count = $service->totalAmount();
        $this->json_success(array( 'count' => $count ));
    }
}
