<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

use \App\Util;

class Item
{
    protected $props;

    public function __construct($props = array())
    {
        if (!is_array($props)) {
            throw new \InvalidArgumentException('Props is not array');
        }
        $this->props = $this->apply_default($props);
    }

    public function apply_default($props)
    {
        if (array_values($props) === $props) {
            $props = array(
                'id' => isset($props[0]) ? $props[0] : null,
                'name' => isset($props[1]) ? $props[1] : null,
                'price' => isset($props[2]) ? $props[2] : 0,
                'groupId' => isset($props[3]) ? $props[3] : null,
                'stock' => isset($props[4]) ? $props[4] : 0,
                'weight' => isset($props[5]) ? $props[5] : 0,
            );
        }
        if (isset($props['price'])) {
            $props['price'] = $this->fixPrice($props['price']);
        }
        $defaults = array(
            'id' => null,
            'name' => null,
            'price' => 0,
            'groupId' => null,
            'stock' => 0,
            'weight' => 0,
        );
        $props = array_merge($defaults, $props);
        return $props;
    }

    private function fixPrice($price)
    {
        $price = Util::zen2han($price);
        $price = str_replace(',', '', $price);
        $price = intval($price);
        return $price;
    }

    public function toString()
    {
        $props = array_map(function ($prop) {
            return mb_ereg_replace("\|", "", $prop);
        }, $this->props);
        $str = implode('|', $props);
        return $str;
    }

    public function id()
    {
        return $this->props['id'];
    }

    public function name()
    {
        return $this->props['name'];
    }

    public function price()
    {
        return $this->props['price'];
    }

    public function groupId()
    {
        return $this->props['groupId'];
    }

    public function stock()
    {
        return $this->props['stock'];
    }

    public function setStock($stock)
    {
        $this->props['stock'] = $stock;
    }

    public function addStock($stock)
    {
        if (($this->props['stock'] + $stock) < 0) {
            return false;
        }
        $this->props['stock'] += $stock;
        return $this->props['stock'];
    }

    public function weight()
    {
        return $this->props['weight'];
    }

    public function find($id)
    {
        $columns = $this->itemList()->findOne($id);
        if ($columns === false) {
            return false;
        }
        $item = new self($columns);
        return $item;
    }

    public function itemList()
    {
        return \App::itemList();
    }

    public function save()
    {
        return $this->itemList()->update($this->getProperties());
    }

    public function getProperties()
    {
        return $this->props;
    }

    public function allocateStock($amount)
    {
        if ($this->props['stock'] < $amount) {
            throw new OutOfStockException(
                "商品名:" . $this->props['name'] . "(在庫" . $this->props['stock'] . ")点"
            );
        }
        $this->props['stock'] -= $amount;
    }
}

