<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class PreviewController extends Controller
{
    public function __construct($type = null, $mode = null)
    {
        if (is_null($type)) {
            $type = 'preview';
        }
        parent::__construct($type, $mode);
    }

    public function process()
    {
        if(!\App::config()->isPreviewMode()) {
            \App::notFound();
        }
        try {
            $method = $this->type . "_" . $this->mode;
            if(method_exists($this, $method)) {
                call_user_func(array($this, $method));
            }
            else {
                \App::notFound();
            }
        } catch (InvalidCartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            $this->redirect_to_cart();
        } catch (\Exception $e) {
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            $this->render('_error.html', $data);
        }
    }

    public function getTemplateParams()
    {
        $data = parent::getTemplateParams();
        $slip = \App::slip();
        $slip_data = $slip->getProperties();
        $data = array_merge($data, $slip_data);
        return $data;
    }

    public function checkout_edit() {
        $data = $this->getTemplateParams();
        $data['error_message'] = "あれもエラー\nこれもエラー";
        $this->render('_regi.html', $data);
    }

    public function checkout_confirm() {
        $data = $this->getTemplateParams();
        $this->render('_regi_conf.html', $data);
    }

    public function checkout_complete() {
        $data = $this->getTemplateParams();
        $data['payment_way_name'] = 'ホゲホゲ';
        $data['payment_way'] = 4;
        $this->render('_regi_comp.html', $data);
    }

    public function cart_show() {
        $data = $this->getTemplateParams();
        $this->render('_cart.html', $data);
    }
}