<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App\Service;


class Checkout
{

    public function checkCart()
    {
        $cart = \App::cart();
        if ($cart->isEmpty()) {
            throw new \App\CartException("カートに商品がありません。");
        }
        try {
            if (!$cart->isValid()) {
                throw new \App\InvalidCartException("カート内に注文できない商品があります。");
            }
        } catch (\Exception $e) {
            // HACK: for php 5.3
        } /* finally */
        {
            $cart->save(\App::session());
            if (isset($e)) {
                throw $e;
            }
        }
    }

    public function createEmptySlip()
    {
        // Copy cart's price and weight to params.
        $cart = \App::cart();
        if ($cart->isEmpty()) {
            throw new \App\CartException("カートに商品がありません。");
        }
        try {
            if (!$cart->isValid()) {
                throw new \App\InvalidCartException("カート内に注文できない商品があります。");
            }
        } catch (\Exception $e) {
            // HACK: for php 5.3
        } /* finally */
        {
            $cart->save(\App::session());
            if (isset($e)) {
                throw $e;
            }
        }

        $slip = \App::slip();
        $slip->setWeight($cart->totalWeight());
        $slip->setSubTotal($cart->subTotalPrice());
        $slip->setTax($cart->tax());
        $slip->setPayment($cart->payment());
    }

    public function createSlip($params)
    {
        if (empty($params)) {
            throw new \UnexpectedValueException('Missing argument');
        }

        // Copy cart's price and weight to params.
        $cart = \App::cart();
        if ($cart->isEmpty()) {
            throw new \App\CartException("カートに商品がありません。");
        }
        try {
            if (!$cart->isValid()) {
                throw new \App\InvalidCartException("カート内に注文できない商品があります。");
            }
        } catch (\Exception $e) {
            // HACK: for php 5.3
        } /* finally */
        {
            $cart->save(\App::session());
            if (isset($e)) {
                throw $e;
            }
        }

        $cart_data = array(
            'weight' => $cart->totalWeight(),
            'subtotal' => $cart->subTotalPrice(),
        );
        $params = array_merge($params, $cart_data);

        $slip = \App::slip();
        // NOTE: Propセット時にValidationExceptionが発生すると失われる値をセットしておく
        $slip->setWeight($cart->totalWeight());
        $slip->setSubTotal($cart->subTotalPrice());
        $slip->setTax($cart->tax());
        $slip->setPayment($cart->payment());
        // NOTE: ValidationExceptionが発生するかもしれない
        $slip->setProperties($params);
        $slip->save(\App::session());
    }

    public function order($params = null)
    {
        $session = \App::session();
        $slip = \App::slip();
        if (!$slip->isValid()) {
            throw new \App\SlipException("無効な伝票です。");
        }

        $cart = \App::cart();
        if ($cart->isEmpty()) {
            throw new \App\CartException("カートに商品がありません。");
        }
        try {
            if (!$cart->isValid()) {
                throw new \App\InvalidCartException("カート内に注文できない商品があります。");
            }
        } catch (\Exception $e) {
            // HACK: for php 5.3
        } /* finally */
        {
            $cart->save(\App::session());
            if (isset($e)) {
                throw $e;
            }
        }

        try {
            $stockAllocator = null;
            if (\App::config()->useStockManagement()) {
                $stockAllocator = \App::stockAllocator(true);
                $stockAllocator->setCart($cart);
                $stockAllocator->setItemModel(\App::item());
                $cart->allocateStock($stockAllocator);
            }

            // 外部決済サイトへ送信
            \App::runCallbacks('sendOrder', $cart, $slip);

            $mailer = \App::mailer();
            $mailer->setConfig(\App::config());
            $mailer->setCart($cart);
            $mailer->setSlip($slip);
            $mailer->setView(\App::view());
            $mailer->send();

            if ($stockAllocator) {
                $stockAllocator->save();
            }
            $slip->save($session);
        } catch (\Exception $e) {
            // 外部決済サイトへの送信をキャンセル
            \App::runCallbacks('cancelOrder', $cart, $slip, $e);
            throw $e;
        }
    }

    public function complete($params = null)
    {
        // Session からロード
        $cart = \App::cart();
        if ($cart->isEmpty()) {
            throw new \App\CartException("カートに商品がありません。");
        }
        $slip = \App::slip();
        if (!$slip->isValid()) {
            throw new \App\SlipException("無効な伝票です。");
        }

        // Session をクリア
        $session = \App::session();
        $session->set('cart', null);
        $session->set('slip', null);
        $session->save();
    }
}
