<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

use App\Util;

class Validator
{
    /*
     * array(
     *      'id' => array(
     *          'REQUIRED' => '商品IDを指定してください。',
     *      ),
     *      'name' => array(
     *          'REQUIRED' => '商品名を指定してください。',
     *      ),
     *      'price' => array(
     *          'REQUIRED' => '価格を指定してください。',
     *          'NUMERIC' => '価格は数値を指定してください。',
     *      ),
     *      'options' => array(
     *          array($this, 'foo') => 'オプションが不正です。',
     *      ),
     * );
     */
    protected $rules;
    protected $errors = array();

    public function __construct($rules = null)
    {
        $this->rules = $rules;
    }

    public function validate($params)
    {
        $this->errors = array();

        foreach ($this->rules as $key => $rule) {
            $required = array_key_exists('REQUIRED', $rule);
            $required_if = array_key_exists('REQUIRED_IF', $rule);
            $value = null;
            if (array_key_exists($key, $params)) {
                $value = $params[$key];
            } else {
                if (!($required || $required_if)) {
                    continue;
                }
            }
            foreach ($rule as $func => $data) {
                $result = true;
                $msg = '';
                $args = null;
                if (is_array($data)) {
                    if (preg_match('/^CUSTOM/', $func) && isset($data['func'])) {
                        $func = $data['func'];
                    }
                    if (isset($data['message'])) {
                        $msg = $data['message'];
                    }
                    if (isset($data['args'])) {
                        $args = $data['args'];
                    }
                } else {
                    $msg = $data;
                }
                if (is_callable($func)) {
                    $result = call_user_func_array($func, array($key, $value, $params, $args));
                } else if (is_string($func)) {
                    $func = Util::camelize(strtoupper($func));
                    if (method_exists($this, $func)) {
                        $result = call_user_func_array(array($this, $func), array($key, $value, $params, $args));
                    }
                }
                if ($result === false) {
                    $this->errors[$key] = is_callable($msg)
                        ? call_user_func_array($msg, array($key, $value, $params, $args))
                        : $msg;
                    break;
                }
            }
        }

        return !$this->hasError();
    }

    public function hasError()
    {
        return count($this->errors) > 0 ? true : false;
    }

    public function setRules($rules)
    {
        $this->rules = $rules;
    }

    public function getRules()
    {
        return $this->rules;
    }

    public function getErrors()
    {
        return $this->errors;
    }

    protected function numeric($key, $value, $params, $args)
    {
        if (is_numeric($value)) {
            return true;
        }
        return false;
    }

    protected function length($key, $value, $params, $args)
    {
        if (!isset($value)) {
            return false;
        }
        $min = 0;
        if (isset($args['min'])) {
            $min = $args['min'];
        }
        $max = 0;
        if (isset($args['max'])) {
            $max = $args['max'];
        }
        if (mb_strlen($value) >= $min
            && mb_strlen($value) <= $max
        ) {
            return true;
        }
        return false;
    }

    protected function email($key, $value, $params, $args)
    {
        return filter_var($value, FILTER_VALIDATE_EMAIL);
    }

    protected function requiredIf($key, $value, $params, $args)
    {
        $refValue = null;
        if (isset($args['key'])) {
            $refKey = $args['key'];
            if (array_key_exists($refKey, $params)) {
                $refValue = $params[$refKey];
            }
        }
        if ($refValue) {
            $result = $this->required($key, $value, $params, $args);
            return $result;
        }
        return true;
    }

    protected function required($key, $value, $params, $args)
    {
        if (!isset($value) || Util::is_empty($value)) {
            return false;
        }
        return true;
    }

    protected function dateFmt($key, $value, $params, $args)
    {
        if(isset($value) && !Util::is_empty($value)) {
            $format = \App::config()->get('date_format', 'Y-m-d');
            $result = date_create_from_format($format, $value) ? true : false; 
            return $result;
        }
        return true;
    }
}

